@@ -6,7 +6,8 @@ use std::{collections::VecDeque, io::ErrorKind, path::Path};
 use tokio::net::UnixStream;
 
 use crate::{
-    Error, Interface, InterfaceAddress, InterfaceProperties, Message, Result, ShowInterfacesMessage,
+    Error, Interface, InterfaceAddress, InterfaceProperties, InterfaceSummary, Message, Protocol,
+    ProtocolDetail, Result, ShowInterfacesMessage, ShowProtocolDetailsMessage,
 };
 
 /// An active connection, on which requests can be executed, and responses
@@ -92,6 +93,27 @@ impl Connection {
         }
     }
 
+    /// Sends a `show interfaces summary` request and returns the parsed response as a
+    /// list of [InterfaceSummary] entries, one each for an interface.
+    pub async fn show_interfaces_summary(&mut self) -> Result<Vec<InterfaceSummary>> {
+        let messages = self.send_request("show interfaces summary").await?;
+
+        // we ignore the 2005 message, and focus only on 1005
+        for message in &messages {
+            // if we get a 1005, we process it and return it
+            if let Message::InterfaceSummary(_) = message {
+                return if let Some(ifcs) = InterfaceSummary::from_enum(message) {
+                    Ok(ifcs)
+                } else {
+                    Err(Error::ParseError(messages))
+                };
+            }
+        }
+
+        // if we didn't encounter any 1005, we return a ParseError
+        Err(Error::ParseError(messages))
+    }
+
     /// Sends a `show interfaces` request and returns the parsed response as a
     /// list of [ShowInterfacesMessage] entries, one each for an interface.
     pub async fn show_interfaces(&mut self) -> Result<Vec<ShowInterfacesMessage>> {
@@ -172,6 +194,103 @@ impl Connection {
         }
     }
 
+    /// Sends a `show protocols [<pattern>]` request and returns the parsed response as a
+    /// list of [InterfaceSummary] entries, one for each protocol.
+    ///
+    /// If `pattern` is specified, results of only those protocols is returned, which
+    /// match the pattern.
+    pub async fn show_protocols(&mut self, pattern: Option<&str>) -> Result<Vec<Protocol>> {
+        let cmd = if let Some(pattern) = pattern {
+            format!("show protocols \"{}\"", pattern)
+        } else {
+            "show protocols".into()
+        };
+        let messages = self.send_request(&cmd).await?;
+
+        // we ignore the 2002 message, and focus only on 1005
+        for message in &messages {
+            // if we get a 1002, we process it and return it
+            if let Message::ProtocolList(_) = message {
+                return if let Some(protocols) = Protocol::from_enum(message) {
+                    Ok(protocols)
+                } else {
+                    Err(Error::ParseError(messages))
+                };
+            }
+        }
+
+        // if we didn't encounter any 1002, we return a ParseError
+        Err(Error::ParseError(messages))
+    }
+
+    /// Sends a `show protocols all [<pattern>]` request and returns the parsed response as a
+    /// list of [ShowProtocolDetailsMessage] entries, one for each protocol instance.
+    ///
+    /// If `pattern` is specified, results of only those protocols is returned, which
+    /// match the pattern.
+    pub async fn show_protocols_details(
+        &mut self,
+        pattern: Option<&str>,
+    ) -> Result<Vec<ShowProtocolDetailsMessage>> {
+        let cmd = if let Some(pattern) = pattern {
+            format!("show protocols all \"{}\"", pattern)
+        } else {
+            "show protocols all".into()
+        };
+        let mut result = vec![];
+        let messages = self.send_request(&cmd).await?;
+
+        // we expect messages to show up as a series of doublets: 1002 and 1006
+        if let Some(mut idx) = messages
+            .iter()
+            .position(|x| matches!(x, Message::ProtocolList(_)))
+        {
+            while idx < messages.len() {
+                // each iteration here means fully going through the pair of 1002 and 1006
+
+                if let Some(protocol) = Protocol::from_enum(&messages[idx]) {
+                    // move index to the next message
+                    idx += 1;
+                    // if we have a valid 1002 ...
+                    if let Some(protocol) = protocol.first() {
+                        // ... check for 1006
+                        if idx == messages.len()
+                            || !matches!(messages[idx], Message::ProtocolDetails(_))
+                        {
+                            // if we're already at the end, or if there's no 1006 after the 1002 we saw
+                            // just a step before, we push the current 1002 without any 1006, and continue
+                            result.push(ShowProtocolDetailsMessage {
+                                protocol: protocol.clone(),
+                                detail: None,
+                            });
+                            continue;
+                        }
+                        // looks like we have a valid 1006, so let's process it
+                        if let Some(detail) = ProtocolDetail::from_enum(&messages[idx]) {
+                            // looks like we got a valid 1006
+                            idx += 1;
+                            result.push(ShowProtocolDetailsMessage {
+                                protocol: protocol.clone(),
+                                detail: Some(detail),
+                            });
+                        } else {
+                            log::error!("conn: failed to parse 1006 message");
+                            return Err(Error::ParseError(messages));
+                        }
+                    }
+                } else {
+                    log::error!("conn: failed to parse 1002 message: {:?}", messages[idx]);
+                    return Err(Error::ParseError(messages));
+                }
+            }
+
+            Ok(result)
+        } else {
+            // No 1002 entries, so empty result
+            Ok(Vec::new())
+        }
+    }
+
     /// Reads a full [Message] from the server, and returns it
     async fn next_message(&mut self) -> Result<Message> {
         // if we have pending messages, return the first one
